@@ -241,6 +241,8 @@ gsize         gdk_pixbuf_get_byte_length     (const GdkPixbuf *pixbuf);
 guchar       *gdk_pixbuf_get_pixels_with_length (const GdkPixbuf *pixbuf,
                                                  guint           *length);
 
+const guint8* gdk_pixbuf_read_pixels         (const GdkPixbuf  *pixbuf);
+GBytes *      gdk_pixbuf_read_pixel_bytes    (const GdkPixbuf  *pixbuf);
 
 
 
@@ -298,12 +300,23 @@ GdkPixbuf *gdk_pixbuf_new_from_data (const guchar *data,
 				     GdkPixbufDestroyNotify destroy_fn,
 				     gpointer destroy_fn_data);
 
+GdkPixbuf *gdk_pixbuf_new_from_bytes (GBytes *data,
+				      GdkColorspace colorspace,
+				      gboolean has_alpha,
+				      int bits_per_sample,
+				      int width, int height,
+				      int rowstride);
+ 
 GdkPixbuf *gdk_pixbuf_new_from_xpm_data (const char **data);
+
+#ifndef GDK_PIXBUF_DISABLE_DEPRECATED
+G_DEPRECATED
 GdkPixbuf* gdk_pixbuf_new_from_inline	(gint          data_length,
 					 const guint8 *data,
 					 gboolean      copy_pixels,
 					 GError      **error);
-       
+#endif
+
 /* Mutations */
 void       gdk_pixbuf_fill              (GdkPixbuf    *pixbuf,
                                          guint32       pixel);
@@ -456,6 +469,7 @@ GdkPixbuf *gdk_pixbuf_apply_embedded_orientation (GdkPixbuf *src);
 
 const gchar * gdk_pixbuf_get_option (GdkPixbuf   *pixbuf,
                                               const gchar *key);
+GHashTable * gdk_pixbuf_get_options (GdkPixbuf   *pixbuf);
 
 
 G_END_DECLS
