@@ -1074,7 +1074,7 @@ MODULE_ENTRY (fill_info) (GdkPixbufFormat *info)
 
 	info->name = "pnm";
 	info->signature = (GdkPixbufModulePattern *) signature;
-	info->description = N_("The PNM/PBM/PGM/PPM image format family");
+	info->description = NC_("image format", "PNM/PBM/PGM/PPM");
 	info->mime_types = (gchar **) mime_types;
 	info->extensions = (gchar **) extensions;
 	info->flags = GDK_PIXBUF_FORMAT_THREADSAFE;
